local options = {
	type='group',
	args = {
		death = {
			name = "death",
			type = 'toggle',
			desc = "Stop event tracking after player death",
			get = function () return Expiration.db.profile.death; end,
			set = function () Expiration.db.profile.death = not Expiration.db.profile.death; end,
		},
		health = {
			name = "health",
			type = 'toggle',
			desc = "Display player health with combat messages",
			get = function () return Expiration.db.profile.health; end,
			set = function () Expiration.db.profile.health = not Expiration.db.profile.health; end,
		},
		event = {
			name = "event",
			type = 'toggle',
			desc = "Include the combat event with the logged message",
			get = function () return Expiration.db.profile.event; end,
			set = function () Expiration.db.profile.event = not Expiration.db.profile.event; end,
		},
		lines = {
			name = "lines",
			type = 'range',
			desc = "Number of lines to track per player",
			min = 5,
			max = 50,
			step = 1,
			get = function () return Expiration.db.profile.lines; end,
  			set = function (value) Expiration.db.profile.lines = value; end,
		},
		filter = {
			name = "filter",
			type = 'group',
			desc = "Settings for event filtration",
			args = {
				list = {
					name = "list",
					type = 'execute',
					desc = "List the current filters",
					func = "FilterList",
				},
				add = {
					name = "add",
					type = 'text',
					desc = "Add a pattern to be filtered out",
					get = false,
					set = "FilterAdd",
					usage = "<pattern>",
				},
				remove = {
					name = "remove",
					type = 'text',
					desc = "Remove a filter",
					get = false,
					set = "FilterRemove",
					validate = function (text) return (Expiration.db.profile.filter[tonumber(text)] ~= nil); end,
					usage = "<pattern number>",
				},
			},
		},
		dest = {
			name = "dest",
			type = 'text',
			desc = "Default report channel",
			get = function () return Expiration.db.profile.dest; end,
			set = function (text) Expiration.db.profile.dest = text; end,
			validate = {"console", "say", "party", "raid", "guild", "officer"},
		},
		report = {
			name = "report",
			type = 'text',
			input = true,
			desc = "Show a report on a given player",
			get = false,
			set = "Report",
			usage = "<name> [dest] [lines]",
		},
	},
};

local events = {
	"CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_HITS",
	"CHAT_MSG_COMBAT_CREATURE_VS_CREATURE_MISSES",
	"CHAT_MSG_COMBAT_CREATURE_VS_PARTY_HITS",
	"CHAT_MSG_COMBAT_CREATURE_VS_PARTY_MISSES",
	"CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS",
	"CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES",
--	"CHAT_MSG_COMBAT_ERROR",
--	"CHAT_MSG_COMBAT_FACTION_CHANGE",
--	"CHAT_MSG_COMBAT_FRIENDLYPLAYER_HITS",
--	"CHAT_MSG_COMBAT_FRIENDLYPLAYER_MISSES",
	"CHAT_MSG_COMBAT_FRIENDLY_DEATH",
--	"CHAT_MSG_COMBAT_HONOR_GAIN",
--	"CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS",
--	"CHAT_MSG_COMBAT_HOSTILEPLAYER_MISSES",
	"CHAT_MSG_COMBAT_HOSTILE_DEATH",
--	"CHAT_MSG_COMBAT_MISC_INFO",
--	"CHAT_MSG_COMBAT_PARTY_HITS",
--	"CHAT_MSG_COMBAT_PARTY_MISSES",
--	"CHAT_MSG_COMBAT_PET_HITS",
--	"CHAT_MSG_COMBAT_PET_MISSES",
	"CHAT_MSG_COMBAT_SELF_HITS",
--	"CHAT_MSG_COMBAT_SELF_MISSES",
--	"CHAT_MSG_COMBAT_XP_GAIN",
	"CHAT_MSG_SPELL_AURA_GONE_OTHER",
	"CHAT_MSG_SPELL_AURA_GONE_SELF",
	"CHAT_MSG_SPELL_BREAK_AURA",
--	"CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF",
	"CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE",
	"CHAT_MSG_SPELL_CREATURE_VS_PARTY_BUFF",
	"CHAT_MSG_SPELL_CREATURE_VS_PARTY_DAMAGE",
	"CHAT_MSG_SPELL_CREATURE_VS_SELF_BUFF",
	"CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE",
--	"CHAT_MSG_SPELL_DAMAGESHIELDS_ON_OTHERS",
	"CHAT_MSG_SPELL_DAMAGESHIELDS_ON_SELF",
--	"CHAT_MSG_SPELL_FAILED_LOCALPLAYER",
	"CHAT_MSG_SPELL_FRIENDLYPLAYER_BUFF",
	"CHAT_MSG_SPELL_FRIENDLYPLAYER_DAMAGE",
	"CHAT_MSG_SPELL_HOSTILEPLAYER_BUFF",
	"CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE",
--	"CHAT_MSG_SPELL_ITEM_ENCHANTMENTS",
	"CHAT_MSG_SPELL_PARTY_BUFF",
--	"CHAT_MSG_SPELL_PARTY_DAMAGE",
--	"CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS",
--	"CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_PARTY_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE",
	"CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS",
	"CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE",
--	"CHAT_MSG_SPELL_PET_BUFF",
--	"CHAT_MSG_SPELL_PET_DAMAGE",
	"CHAT_MSG_SPELL_SELF_BUFF",
	"CHAT_MSG_SPELL_SELF_DAMAGE"
};

Expiration = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0");
Expiration:RegisterChatCommand("/expiration", options);
Expiration:RegisterDB("ExpirationDB");
Expiration:RegisterDefaults("profile", {
		death = true,
		health = true,
		lines = 10,
		dest = "console",
		filter = { },
	});

Expiration.players = { };
Expiration.history = { };
Expiration.deathevents = { };

function Expiration:OnEnable()

	-- Register Party/Raid Events
	self:RegisterBucketEvent({"PARTY_MEMBERS_CHANGED", "RAID_ROSTER_UPDATE"}, 1, "UpdatePlayers");

	-- Register Combat Events
	for _, event in ipairs(events) do
		self:RegisterEvent(event, "CombatEvent");
	end
	
	self:UpdatePlayers();

end

function Expiration:FormatName(name)

	if not type(name) == "string" then return nil; end
	
	return strupper(strsub(name,1,1))..strlower(strsub(name,2));

end

function Expiration:FilterList()

	self:Print(getn(self.db.profile.filter).." Filtered Strings:");

	for n, pattern in ipairs(self.db.profile.filter) do
		self:Print(n..". \'"..pattern.."\'");
	end

end

function Expiration:FilterAdd(text)

	-- ensure we aren't duplicating anything
	for index, filter in ipairs(self.db.profile.filter) do
		if strmatch(strlower(text), strlower(filter)) then
			self:Print("Failed to add filter: \'"..text.."\' is already filtered by "..index..". \'"..filter.."\'");
			return;
		end
	end

	-- add the string
	tinsert(self.db.profile.filter, text);

	-- feedback!
	self:Print("Added "..getn(self.db.profile.filter)..". \'"..text.."\' to filter list.");

end

function Expiration:FilterRemove(index)

	index = tonumber(index);
	
	if not index then return; end

	-- feedback!
	self:Print("Removed "..index..". \'"..self.db.profile.filter[index].."\'");

	-- remove the string
	tremove(self.db.profile.filter, index);

end

function Expiration:UpdatePlayers()

	-- clear out the table of players
	for player in pairs(self.players) do self.players[player] = nil; end

	-- Player is in a raid
	if GetNumRaidMembers() ~= 0 then

		-- rebuild list of raid members
		for i = 1, 40 do
			local unit = "raid"..i;
			if UnitExists(unit) then self.players[UnitName(unit)] = unit; end
		end

	-- Player is in a party
	elseif GetNumPartyMembers() ~= 0 then

		-- rebuild list of party members
		for i = 1, 4 do
			local unit = "party"..i;
			if UnitExists(unit) then self.players[UnitName(unit)] = unit; end
		end

	end
	
	-- Add the player
	self.players[UnitName("player")] = "player";

	-- clean out any event data that doesn't correspond to someone in the raid
	for player in pairs(self.history) do
		if not self.players[player] then
			self.history[player] = nil;
			self.deathevents[player] = nil;
		end
	end

	-- add tables for players not in history
	for player in pairs(self.players) do
		if not self.history[player] then
			self.history[player] = { };
			self.deathevents[player] = 0;
		end
	end

end

function Expiration:AddEvent(name, event)

	if not name or not event then return; end
	if not self.players[name] then return; end
	
	if self.db.profile.death then
		if UnitIsDeadOrGhost(self.players[name]) then
			if self.deathevents[name] < 5 then
				self.deathevents[name] = self.deathevents[name] + 1;
			else
				return;
			end
		else
			self.deathevents[name] = 0;
		end
	end

	if getn(self.history[name]) >= self.db.profile.lines then
		tremove(self.history[name], 1);
	end

	local message = format("%05.2f ", math.fmod(GetTime(), 60));

	if self.db.profile.health then
		local health = UnitHealth(self.players[name]);
		local pcthp = health/UnitHealthMax(self.players[name]);
		local red, green;
		
		if ( pcthp > 0.5 ) then
			red = floor(510*(1 - pcthp));
			green = 255;
		else
			red = 255;
			green = floor(510*pcthp);
		end

		message = message..format("[|cff%02X%02X00%d %d%%|r] ", red, green, health, floor(100*pcthp));
	end
	
	message = message..event;

	tinsert(self.history[name], message);

end

function Expiration:CombatEvent(message)

	if not message then return; end

	local name = UnitName("player");
	local trailer = "";

	-- filter
	for _, filter in ipairs(self.db.profile.filter) do
		if strfind(strlower(message), strlower(filter)) then return; end
	end

	if self.db.profile.event then
		trailer = " ("..AceLibrary("AceEvent-2.0").currentEvent..")";
	end

	-- search for you
	if strfind(" "..message.." ", " [Yy]ou ") then self:AddEvent(name, message..trailer); end

	-- search for players
	for player in pairs(self.players) do
		if strfind(" "..message.." ", player) or
		   strfind(" "..message.." ", player.."'s") then
			self:AddEvent(player, message..trailer);
		end
	end

end

function Expiration:Report(player, dest, lines)

	-- self:Print("Expiration:Report("..(player or "nil")..", "..(dest or "nil")..", "..(lines or "nil")..")");

	if not player then
		self:Print("Enter a player to report on.");
		return;
	end

	player = self:FormatName(player);

	if not self.history[player] then
		self:Print("No events found for \'"..player.."\'");
		return;
	end

	if not dest then dest = self.db.profile.dest; end

	if type(dest) == "number" then
		lines = dest;
		dest = self.db.profile.dest;
	else
		dest = strlower(dest);
		if dest ~= "console" and dest ~= "say" and dest ~= "party" and dest ~= "raid" and dest ~= "guild" and dest ~= "officer" then
			self:Print(dest .. " is not a valid destination channel.");
			return;
		end
	end

	if not lines then lines = self.db.profile.lines; end
	if lines < 1 then lines = 1; end
	if lines > self.db.profile.lines then lines = self.db.profile.lines; end
	if lines > getn(self.history[player]) then lines = getn(self.history[player]); end

	local print;

	if dest == "console" then
		print = function (text) self:Print(text); end;
	else
	
		-- Only report chat failures once
		if dest == "party" and GetNumPartyMembers() == 0 then
			SendChatMessage("placeholder", "PARTY");
			return;
		elseif dest == "raid" and GetNumRaidMembers() == 0 then
			SendChatMessage("placeholder", "RAID");
			return;
		elseif (dest == "guild" or dest == "officer") and not IsInGuild() then
			SendChatMessage("placeholder", "GUILD");
			return;
		end

		print = function (text)
			-- strip color info, not safe to send off client
			text = gsub(text, "|c%x%x%x%x%x%x%x%x", "");
			text = gsub(text, "|r", "");
			SendChatMessage(text, dest);
		end;
	end

	print("Expiration report for "..player.." (Last "..lines.." events)");

	for line = getn(self.history[player]) - lines + 1, getn(self.history[player]) do
		print(self.history[player][line]);
	end

end
